@@ -4,11 +4,11 @@ use std::process::Command;
 
 use crate::{
     lint_message::LintMessage,
-    log_utils::log_files,
+    log_utils::{ensure_output, log_files},
     path::{path_relative_from, AbsPath},
 };
 use anyhow::{anyhow, bail, ensure, Context, Result};
-use glob::Pattern;
+use glob::{MatchOptions, Pattern};
 use log::{debug, info};
 
 pub struct Linter {
@@ -23,13 +23,23 @@ pub struct Linter {
 fn matches_relative_path(base: &Path, from: &Path, pattern: &Pattern) -> bool {
     // Unwrap ok because we already checked that both paths are absolute.
     let relative_path = path_relative_from(from, base).unwrap();
-    pattern.matches(relative_path.to_str().unwrap())
+    pattern.matches_with(
+        relative_path.to_str().unwrap(),
+        MatchOptions {
+            case_sensitive: true,
+            // Explicitly set this option to true. Most unix implementations do
+            // not allow `*` to match across path segments, so the default
+            // (false) behavior is unexpected for people.
+            require_literal_separator: true,
+            require_literal_leading_dot: false,
+        },
+    )
 }
 
 impl Linter {
     fn get_config_dir(&self) -> &Path {
         // Unwrap is fine here because we know this path is absolute and won't be `/`
-        self.config_path.as_pathbuf().parent().unwrap()
+        self.config_path.parent().unwrap()
     }
 
     fn get_matches(&self, files: &[AbsPath]) -> Vec<AbsPath> {
@@ -37,14 +47,15 @@ impl Linter {
         files
             .iter()
             .filter(|name| {
-                self.include_patterns.iter().any(|pattern| {
-                    matches_relative_path(config_dir, name.as_pathbuf().as_path(), pattern)
-                })
+                self.include_patterns
+                    .iter()
+                    .any(|pattern| matches_relative_path(config_dir, name, pattern))
             })
             .filter(|name| {
-                !self.exclude_patterns.iter().any(|pattern| {
-                    matches_relative_path(config_dir, name.as_pathbuf().as_path(), pattern)
-                })
+                !self
+                    .exclude_patterns
+                    .iter()
+                    .any(|pattern| matches_relative_path(config_dir, name, pattern))
             })
             .cloned()
             .collect()
@@ -54,7 +65,6 @@ impl Linter {
         let tmp_file = tempfile::NamedTempFile::new()?;
         for matched_file in &matched_files {
             let name = matched_file
-                .as_pathbuf()
                 .to_str()
                 .ok_or_else(|| anyhow!("Could not convert path to string."))?;
             writeln!(&tmp_file, "{}", name)?;
@@ -75,7 +85,11 @@ impl Linter {
             "Running linter {}: {} {}",
             self.code,
             program[0],
-            arguments.join(" ")
+            arguments
+                .iter()
+                .map(|arg| "'".to_owned() + arg + "'")
+                .collect::<Vec<String>>()
+                .join(" ")
         );
 
         let start = std::time::Instant::now();
@@ -90,6 +104,7 @@ impl Linter {
                 )
             })?;
         debug!("Linter {} took: {:?}", self.code, start.elapsed());
+        ensure_output("Linter command", &command)?;
 
         if !&command.status.success() {
             let stderr = std::str::from_utf8(&command.stderr)?;
@@ -182,3 +197,21 @@ impl Linter {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use std::path::PathBuf;
+
+    use super::*;
+
+    // Check that `*` does not match across path segments.
+    #[test]
+    fn test_glob_with_separator() -> Result<()> {
+        assert!(!matches_relative_path(
+            &PathBuf::from(""),
+            &PathBuf::from("foo/bar/baz"),
+            &Pattern::new("foo/b*")?,
+        ));
+        Ok(())
+    }
+}
